from PyDSTool import *

print '-------- Test: InterpolateTable'
xnames = ['x1', 'x2']
timeData = array([0.1, 1.1, 2.1])
x1data = array([10.2, -1.4, 4.1])
x2data = array([0.1, 0.01, 0.4])
xData = makeDataDict(xnames, [x1data, x2data])
itableArgs = {}
itableArgs['tdata'] = timeData
itableArgs['ics'] = xData
itableArgs['name'] = 'interp'
interptable = InterpolateTable(itableArgs)
itabletraj = interptable.compute('itable')
print "itabletraj(0.4, 'x1') = ", itabletraj(0.4, 'x1')
print "                         (x1 should be 6.72)"
print "itabletraj(1.1) = ", itabletraj(1.1)

print '-------- Test: LookupTable'
ltableArgs = copy(itableArgs)
ltableArgs['name'] = 'lookup'
lookuptable = LookupTable(ltableArgs)
ltabletraj = lookuptable.compute('ltable')
print "ltabletraj(1.1) = ", ltabletraj(1.1)
print "             (x2 should be 0.01)\n"
print "ltabletraj(0.4) ="
try:
    ltabletraj(0.4)
except ValueError, e:
    print "... ltabletraj(0.4) did not succeed because ", e
    print "(0.4 is not in the tdata array)"


print '\n-------- Test: ODE system'

fvarspecs = {"w": "k*w + a*itable + sin(t) + myauxfn1(t)*myauxfn2(w)",
           'aux_wdouble': 'w*2 + globalindepvar(t)',
           'aux_other': 'myauxfn1(2*t) + initcond(w)'}
fnspecs = {'myauxfn1': (['t'], '2.5*cos(3*t)'),
             'myauxfn2': (['w'], 'w/2')}
DSargs = {'tdomain': [0.1,2.1],
          'pars': {'k':2, 'a':-0.5},
          'inputs': {'itable' : interptable.variables['x1']},
          'auxvars': ['aux_wdouble', 'aux_other'],
          'algparams': {'init_step':0.01, 'strict':False},
          'checklevel': 2,
          'name': 'ODEtest',
          'fnspecs': fnspecs,
          'varspecs': fvarspecs
          }
testODE = Vode_ODEsystem(DSargs)
print "params set => ", testODE.pars
print "DS defined? => ", testODE.defined
print "testODE.set(...)"
testODE.set(ics={'w':3.0},
                tdata=[0.11,2.1])
print "testtraj = testODE.compute('test1')"
testtraj = testODE.compute('test1')
print "DS defined now? => ", testODE.defined
print "testtraj(0.5) => ", testtraj(0.5)
print "testODE.diagnostics.showWarnings() => "
print testODE.diagnostics.showWarnings()
print "\ntestODE.indepvariable.depdomain => ", testODE.indepvariable.depdomain
print "testtraj(0.2, 'aux_other') => ", \
    testtraj(0.2, 'aux_other')

print "\nNow adding a terminating co-ordinate threshold event..."
ev_args = {'name': 'threshold',
           'eventtol': 1e-4,
           'eventdelay': 1e-5,
           'starttime': 0,
           'active': True,  # = default
           'term': True,
           'precise': True  # = default
           }
thresh_ev = Events.makePythonStateZeroCrossEvent('w', 20, 1, ev_args)
testODE.eventstruct.add(thresh_ev)
print "Recomputing trajectory:"
print "traj2 = testODE.compute('test2')"
traj2 = testODE.compute('test2')
print "\ntestODE.diagnostics.showWarnings() => "
print testODE.diagnostics.showWarnings()
print "\ntestODE.indepvariable.depdomain => ", testODE.indepvariable.depdomain


print '\n-------- Test: Explicit functional trajectory'
print "Explicit functional trajectory 'sin_gen' computes sin(t*speed)"

print "Make 'xdomain' argument smaller than known limits for sine wave: [-1.001, 0.7]"
args = {'tdomain': [-50, 50],
        'pars': {'speed': 1},
        'xdomain': {'s': [-1., 0.7]},
        'name': 'sine',
        'globalt0': 0.4,
        'pdomain': {'speed': [0, 200]},
        'varspecs': {'s': "sin(globalindepvar(t)*speed)"}}
sin_gen = ExplicitFnGen(args)
sintraj = sin_gen.compute('sinewave')
print "Using alternative form of call (specific names in specific order)..."
print "sintraj(0.0, ['s'], checklevel=2) = ", sintraj(0.0, ['s'], checklevel=2)
try:
    print "sintraj(0.8, checklevel=2) \n ", sintraj(0.8, checklevel=2)
except ValueError, e:
    print "... correctly raised error: ", e
print "Set limits properly now, to [-1., 1.] ..."
print "sin_gen.set({'xdomain': {'s': [-1., 1.]}})"
sin_gen.set(xdomain={'s': [-1., 1.]})
sintraj2 = sin_gen.compute('sinewave2')
print "sintraj2(0.8, checklevel=2) = ", sintraj2(0.8, checklevel=2)